'use strict';

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _strategies = require('./strategies');

var _strategies2 = _interopRequireDefault(_strategies);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('promisePoller');

var DEFAULTS = {
  strategy: 'fixed-interval',
  retries: 5
};

var pollerCount = 0;

function promisePoller() {
  var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

  if (typeof options.taskFn !== 'function') {
    throw new Error('No taskFn function specified in options');
  }

  Object.keys(DEFAULTS).forEach(function (option) {
    return options[option] = options[option] || DEFAULTS[option];
  });
  options.name = options.name || 'Poller-' + pollerCount++;
  debug('Creating a promise poller "' + options.name + '" with interval=' + options.interval + ', retries=' + options.retries);

  if (!_strategies2.default[options.strategy]) {
    throw new Error('Invalid strategy "' + options.strategy + '". Valid strategies are ' + Object.keys(_strategies2.default));
  }
  var strategy = _strategies2.default[options.strategy];
  debug('(' + options.name + ') Using strategy "' + options.strategy + '".');
  var strategyDefaults = strategy.defaults;
  Object.keys(strategyDefaults).forEach(function (option) {
    return options[option] = options[option] || strategyDefaults[option];
  });

  debug('(' + options.name + ') Options:');
  Object.keys(options).forEach(function (option) {
    debug('    "' + option + '": ' + options[option]);
  });

  return new _bluebird2.default(function (resolve, reject) {
    var retriesRemaining = options.retries;
    var rejections = [];

    if (options.masterTimeout) {
      setTimeout(function () {
        return reject('master timeout');
      }, options.globalTimeout);
    }

    function poll() {
      var taskPromise = _bluebird2.default.resolve(options.taskFn());

      if (options.timeout) {
        taskPromise = taskPromise.timeout(options.timeout);
      }

      taskPromise.then(function (result) {
        debug('(' + options.name + ') Poll succeeded. Resolving master promise.');
        resolve(result);
      }, function (err) {
        rejections.push(err);
        if (typeof options.progressCallback === 'function') {
          options.progressCallback(retriesRemaining, err);
        }

        if (! --retriesRemaining) {
          debug('(' + options.name + ') Maximum retries reached. Rejecting master promise.');
          reject(rejections);
        } else {
          debug('(' + options.name + ') Poll failed. ' + retriesRemaining + ' retries remaining.');

          var nextInterval = strategy.getNextInterval(options.retries - retriesRemaining, options);

          debug('(' + options.name + ') Waiting ' + nextInterval + 'ms to try again.');
          _bluebird2.default.delay(nextInterval).then(poll);
        }
      });
    }

    poll();
  });
}

module.exports = promisePoller;